package com.jdbernard.teammaker

import groovy.beans.Bindable
import groovy.swing.SwingBuilder
import java.awt.GridBagConstraints as GBC
import java.awt.GridBagLayout
import java.awt.BorderLayout
import java.awt.event.MouseEvent
import javax.swing.DefaultListModel
import javax.swing.JFrame
import javax.swing.JOptionPane
import org.apache.log4j.Logger

public class TeamMaker {

    /* ======== MODEL ======== */

    public static def swing = new SwingBuilder()
    public static final String version = "0.3"

    def frame
    def team1List
    List team1Players
    def team2List
    List team2Players
    def team1WinsButton
    def team2WinsButton
    def newGameButton
    def sittingList
    List sittingPlayers

    def inGamePlayerDropMenu
    def sittingPlayerDropMenu

    protected Logger log = Logger.getLogger(getClass())

    PlayerChooser playerChooser = new WeightedChooser(hardLimit: 2)
    OddsCalculator oddsCalculator = new OddsCalculator()

    class Observables {
        @Bindable boolean inGame = false
        @Bindable int teamSize = 4
        @Bindable Player popupPlayer = null
    }
    def model = new Observables()



    /* ======== VIEW ======== */

        /*
           +--------------------------------------------------+
           | JDB TeamMaker v0.2                          _[]X |
           +--------------------------------------------------+
           |                                                  |
           | +-Current-Game--------------+ +-Sitting-Players+ |
           | |                           | |                | |
           | | Team 1          Team 2    | | Jeptha       3 | |
           | |   Derrick         Eric    | | Jack         2 | |
           | |   Shane           Brandon | | Sam          2 | |
           | |   Shane           Darren  | | Greg         2 | |
           | |   Christian       Brett   | | Frank        1 | |
           | +---------------------------+ | Chris D      1 | |
           |                               | Joshua       1 | |
           |                               |                | |
           |                               |                | |
           |                               |                | |
           |                               |                | |
           |                               |                | |
           |                               +----------------+ |
           +--------------------------------------------------+

         */

    private void init() {
        
        inGamePlayerDropMenu = swing.popupMenu(/*'Player Options'*/) {
            menuItem('Bench Player',/* enabled: bind { !model.inGame },*/
                actionPerformed: { benchPlayer(model.popupPlayer) })
        }

        sittingPlayerDropMenu = swing.popupMenu(/*'Player Options'*/) {
            menuItem('Assign to Team 1', /*enabled: bind { !model.inGame },*/
                actionPerformed: { assignPlayer(model.popupPlayer, team1List) })

            menuItem('Assign to Team 2', /*enabled: bind { !model.inGame },*/
                actionPerformed: { assignPlayer(model.popupPlayer, team2List) })

            menuItem('Set Games Sat',
                actionPerformed: { setGamesSat(model.popupPlayer) })
        }

        frame = swing.frame(title: "JDB TeamMaker v$version",
            size: [600, 400], locationRelativeTo: null,
            defaultCloseOperation: JFrame.EXIT_ON_CLOSE) {

            panel() {
                gridBagLayout()

                panel(constraints: gbc(gridx: 0, gridy: 0, fill: GBC.BOTH,
                        insets: [5, 5, 5, 0], weightx: 3, weighty: 1),
                    border: titledBorder(title: 'Current Game')) {
                    
                    gridBagLayout()

                    label('Team A', constraints: gbc(gridx: 0, gridy: 0,
                        fill: GBC.BOTH, insets: [5, 5, 0, 0]))

                    label('Team B', constraints: gbc(gridx: 1, gridy: 0, 
                        fill: GBC.BOTH, insets: [5, 5, 0, 5]))
                    
                    def teamListRenderer = new PlayerListCellRenderer(this,
                        showStats: false, colored: false)

                    team1List = list(cellRenderer: teamListRenderer,
                        constraints: gbc(gridx: 0, gridy: 1, fill: GBC.BOTH,
                        insets: [5, 5, 0, 0], weightx: 2, weighty: 2),
                        model: new DefaultListModel(),
                        mouseClicked: { evt ->
                            if (evt.button != MouseEvent.BUTTON3) return
                            showInGamePlayerDropMenu(evt)
                        })

                    team1WinsButton = button('Team A Wins',
                        constraints: gbc(gridx: 0, gridy: 2, anchor:
                            GBC.CENTER, insets: [5, 5, 5, 0]),
                        enabled: bind { model.inGame },
                        actionPerformed: { declareLoser(team2List)})

                    team2List = list(cellRenderer: teamListRenderer,
                        constraints: gbc(gridx: 1, gridy: 1, fill: GBC.BOTH,
                        insets: [5, 5, 0, 5], weightx: 2, weighty: 2),
                        model: new DefaultListModel(),
                        mouseClicked: { evt ->
                            if (evt.button != MouseEvent.BUTTON3) return
                            showInGamePlayerDropMenu(evt)
                        })

                    team2WinsButton = button('Team B Wins',
                        constraints: gbc(gridx: 1, gridy: 2,
                            anchor: GBC.CENTER, insets: [5, 5, 5, 5]),
                        enabled: bind { model.inGame },
                        actionPerformed: { declareLoser(team1List)} )

                }

                scrollPane(constraints: gbc(gridx: 1, gridy: 0, gridwidth: 2,
                        fill: GBC.BOTH, insets: [5, 5, 0, 5], weighty: 1),
                    border: titledBorder(title: 'Sitting Players')) {

                    sittingList = list(cellRenderer: new PlayerListCellRenderer(
                            this, showStats: true, colored: true),
                            model: new SortedPlayerModel(),
                            mouseClicked: { evt ->
                                if (evt.button != MouseEvent.BUTTON3) return
                                showSittingPlayerDropMenu(evt)
                            })
                }

                button('Next Game', constraints: gbc(gridx: 0, gridy: 1,
                    anchor: GBC.CENTER, insets: [5, 5, 5, 0]),
                    enabled: bind { !model.inGame},
                    actionPerformed: { newGame() })

                button('Add Player', constraints: gbc(gridx: 1, gridy: 1,
                    anchor: GBC.CENTER, insets: [5, 5, 5, 0]),
                    actionPerformed: { addPlayer() })

                newGameButton = button('Delete Player',
                    constraints: gbc(gridx: 2, gridy: 1, anchor: GBC.CENTER,
                        insets: [5, 5, 5, 5]),
                    actionPerformed: {
                        sittingList.model.removeElement(
                            sittingList.selectedValue) })

            }
        }

    }


    

    /* ======== CONTROLLER ======== */

    public static void main(String[] args) {
        def inst = new TeamMaker()
        inst.frame.show()
    }

    TeamMaker() {
        init()
    }

    public int getSpotsOpen() {
        def spots =  (model.teamSize - team1List.model.size()) +
                     (model.teamSize - team2List.model.size())
        return (spots == 0 ? model.teamSize : spots)
    }

    private void addPlayer() {
        def name = JOptionPane.showInputDialog(frame,
            "Enter the new player's name: ", "New Player...",
            JOptionPane.QUESTION_MESSAGE)

        if (!name) return

        def player = new Player()
        player.name = name
        player.gamesSat = 0
        sittingList.model.addElement(player)

        // recalculate odds
        def players = sittingList.model.collect { it }
        swing.doOutside {
            oddsCalculator.recalculate(players, getSpotsOpen(), playerChooser)
            swing.edt { sittingList.repaint() }
        }
    }

    private void newGame() {
        if (team1List.model.size() < model.teamSize)
            populate(team1List)

        if (team2List.model.size() < model.teamSize)
            populate(team2List)

        recalculateOdds()

        model.inGame = true

    }

    private void declareLoser(def teamList) {
        sittingList.model.each { it.gamesSat++ }
        teamList.model.each { player ->
            player.gamesSat = 0
            sittingList.model.addElement(player)
        }

        recalculateOdds()

        teamList.model.clear()
        teamList.repaint()
        sittingList.repaint()
        model.inGame = false
    }

    private void recalculateOdds() {
        def players = sittingList.model.collect { it }
        swing.doOutside {
            oddsCalculator.recalculate(players, getSpotsOpen(), playerChooser)
            swing.edt { sittingList.repaint() }
        }

    }

    private void populate(def teamList) {
        while (teamList.model.size() < model.teamSize) {
            def player = playerChooser.choose(sittingList.model)
            teamList.model.addElement(player)
            sittingList.model.removeElement(player)
        }
    }

    private benchPlayer(Player p) {
        team1List.remove(p)
        sittingList.model.addElement(p)
        recalculateOdds()
    }

    private assignPlayer(Player p, def teamList) {
        sittingList.model.removeElement(p)
        teamList.model.addElement(p)
        recalculateOdds()
    }

    private void showSittingPlayerDropMenu(def evt) {
        int index = sittingList.locationToIndex(evt.point)
        sittingList.selectedIndex = index
        this.model.popupPlayer = sittingList.model.getElementAt(index)

        sittingPlayerDropMenu.show(sittingList, evt.point.@x, evt.point.@y)
    }

    private void showInGamePlayerDropMenu(def evt) {
        def teamList = evt.source
        int index = teamList.locationToIndex(evt.point)
        teamList.selectedIndex = index
        this.model.popupPlayer = teamList.model.getElementAt(index)

        inGamePlayerDropMenu.show(teamList, evt.point.@x, evt.point.@y)
    }

    private void setGamesSat(Player p) {
        String value = JOptionPane.showInputDialog(frame,
            "How many games has ${p.name} sat?", "Set Games Sat.",
            JOptionPane.QUESTION_MESSAGE)
        int newGames

        sittingList.model.removeElement(p)

        if (!value) return
        try { newGames = value.toInteger() }
        catch (Throwable t) {
            JOptionPane.showMessageDialog(frame,
                "The value $value is not a valid number.",
                "Invalid Input", JOptionPane.ERROR_MESSAGE)
            setGamesSat(p)
            return
        }
        
        p.gamesSat = newGames
        if (!sittingList.model.set.contains(p)) sittingList.model.addElement(p)

        recalculateOdds()
    }
}
